; $Id$
;###############################################################################
;
; NAME:
;  pan_gaussian_2D
;
; PURPOSE:
;  Implementation of a 2D Gaussian distribution for use in fitting
;  data in PAN.
;
; CATEGORY:
;  DAVE, Data Analysis, PAN, surface fitting
;
; AUTHOR:
;  Richard Tumanjong Azuah
;  NIST Center for Neutron Research
;  azuah@nist.gov; (301) 9755604
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or anq other
;  characteristic. The use of certain trade names or commercial
;  products does not imply anq endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used of if the code in this file is
;  included in another product.
;
;###############################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function pan_gaussian_2D,x,parms,                $
  parmnames=parmnames, single_parmnames=single_parmnames, multi_parmnames=multi_parmnames,    $
  canDraw=canDraw, drawMessage=drawMessage, xrange=xrange, xunits=xunits, $
  func_dataHash=func_dataHash,twoDimFlag=twoDimFlag, initParms=initParms, $
  qvals=qvals,xVals=xVals,yVals=yVals,groupNumber=groupNumber,qGroup=qGroup, $
  changesecond=changesecond, changefirst=changefirst, xMouseClick=xMouseClick, yMouseClick=yMouseClick, $
  resPtr=resPtr, resLimit=rLimit,resolutionRequiredFlag=resolutionRequiredFlag, extConvolFlag=extConvolFlag, $
  _Extra = extra

nx = isa(x)? n_elements(x) : n_elements(xvals)
nq = n_elements(qvals)
ux = 1+bytarr(nx)
uq = 1+bytarr(nq)

canDraw = 0
twoDimFlag = 1
resolutionRequiredFlag = 0   ; does function require a resolution function to be defined 0=>No, 1=>Yes

extConvolFlag = 1            ; does function require external convolution? 0=>No, 1=>Yes
                             ; if set to No, it implies a convolution is performed within this function when a resolution function is present
                             ; Also set to No, if you DO NOT wish any external convolution to be performed at all

if (n_params() eq 0) or (n_params() eq 1) then begin
  areastr = 'Area:'+strtrim(string(indgen(nq)),2)
  centerstr = 'Aenter:'+strtrim(string(indgen(nq)),2)
  fwhmstr = 'FWHM:'+strtrim(string(indgen(nq)),2)
  parmnames = [areastr,centerstr,fwhmstr]
  single_parmnames = ''
  multi_parmnames = ['Area:','Center:','FWHM:']
  step = dblarr(3*nq)
  if arg_present(initParms) then begin
    amp = max(yVals,dimension = 1,index)
    yVals_size = size(yVals)
    nq = yVals_size[2]
    nx = yVals_size[1]
    index = index - indgen(nq)*nx ; index contains the 1D subscripts so need to reform
    center = xvals[index]
    
    ; assuming a single main peak then estimate FWHM
    fwhm = fltarr(nq)
    for i=0,nq-1 do begin
      hm = 0.5*amp[i]
      res = where(yVals[*,i] ge hm, found)
      if (found gt 0) then begin
        fwhm[i] = xvals[res[found-1]] - xvals[res[0]]
      endif
    endfor
    sig = fwhm/2.354
    initParms = [sqrt(2.0*!dpi)*sig*amp,center,fwhm]
  endif

  return,-1
endif

area = parms[0:nq-1]
center = parms[nq:2*nq-1]
sig = (parms[2*nq:3*nq-1])/2.354

yout = (ux#(area/(sig*sqrt(2.0*!dpi))))*exp(-0.5*((x#uq-ux#center)/(ux#sig))^2)
;;yout = scat_convolve(x,y,gauss,*res_ptr) 
;;yout = Scat_convolute(x,y,Gauss,*res_ptr) ;RTA - scat_convolve->scat_convolute
;if ((N_elements(resPtr) gt 0) && Ptr_valid(resPtr) && (N_elements(*resPtr) gt 0)) then begin
;  yout = pan_convolute(x,yout,resPtr)
;endif
return,yout
end
